#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "stm8s.h"
#include "usrlib-uart.h"

#define fMASTER    16000000  // 16MHz
#define TIM3_FREQ  10        // 10Hz
#define TIM3_PRSC  TIM3_PRESCALER_512
#define TIM3_ARR   ((u16)(fMASTER/512/TIM3_FREQ)-1)   // 16MHz/512/10-1=3124

char UARTstrTX[100];

void main(void)
{
	CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSE, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE);
	
	TIM3_DeInit();
	TIM3_TimeBaseInit(TIM3_PRSC,TIM3_ARR); // 0.1sec timer
	TIM3_ITConfig(TIM3_IT_UPDATE, ENABLE);
	TIM3_Cmd(ENABLE);

  ADC1_DeInit();
  ADC1_Init(ADC1_CONVERSIONMODE_CONTINUOUS, ADC1_CHANNEL_9, ADC1_PRESSEL_FCPU_D4, ADC1_EXTTRIG_TIM, DISABLE, ADC1_ALIGN_RIGHT, ADC1_SCHMITTTRIG_CHANNEL9, DISABLE);
	ADC1_ITConfig(ADC1_IT_AWDIE, ENABLE);
	ADC1_AWDChannelConfig(ADC1_CHANNEL_9, ENABLE);
	ADC1_SetHighThreshold(800);
	ADC1_SetLowThreshold(200);
	ADC1_Cmd(ENABLE);
	ADC1_StartConversion();

	UART2_DeInit();
  UART2_Init((u32)115200, UART2_WORDLENGTH_9D, UART2_STOPBITS_1, UART2_PARITY_ODD, UART2_SYNCMODE_CLOCK_DISABLE, UART2_MODE_TXRX_ENABLE);
  UART2_ITConfig(UART2_IT_RXNE_OR, ENABLE);
	UART2_Cmd(ENABLE);

	GPIO_Init(GPIOD, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_FAST); // LED

  enableInterrupts();

	while(1){ } /*Main loop*/
}

u16 adc;
float mV;

void Tim3Update(void) interrupt 15 // 0.1Sec interrupt
{
	GPIO_WriteReverse(GPIOD,GPIO_PIN_0); // LED flash
	adc=ADC1_GetConversionValue();	// get ADC value
	mV=(float)adc/1024.0*3300.0;	// convert to voltage
	sprintf(UARTstrTX,"ADC=%4d mV=%4d\r\n",(int)adc,(int)mV);
	SerialPutString(UARTstrTX);
	TIM3_ClearFlag(TIM3_FLAG_UPDATE);
	return;
}

void AWD(void) interrupt 22 // analog watch dog interrupt
{
	adc=ADC1_GetConversionValue();	// get ADC value
	mV=(float)adc/1024.0*3300.0;	// convert to voltage
	sprintf(UARTstrTX,"ADC=%4d mV=%4d  WATCHDOG ALERT!!!\r\n",(int)adc,(int)mV);
	SerialPutString(UARTstrTX);
	ADC1_ClearFlag(ADC1_FLAG_AWD);
}

#ifdef USE_FULL_ASSERT
void assert_failed(uint8_t* file, uint32_t line)
{ 
  while (1)  {  }
}
#endif
